
From: NeilBrown <neilb@cse.unsw.edu.au>

The GRACE and NOGRACE errors should bump the sequence id on open.  So we delay
the handling of these errors until nfsd4_process_open2, at which point we've
set the open owner, so the encode routine will be able to bump the sequence
id.

Signed-off-by: J. Bruce Fields <bfields@citi.umich.edu>
Signed-off-by: Neil Brown <neilb@cse.unsw.edu.au>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 fs/nfsd/nfs4proc.c  |    6 ------
 fs/nfsd/nfs4state.c |    6 ++++++
 2 files changed, 6 insertions(+), 6 deletions(-)

diff -puN fs/nfsd/nfs4proc.c~nfsd4-err_grace-should-bump-seqid-on-open fs/nfsd/nfs4proc.c
--- 25/fs/nfsd/nfs4proc.c~nfsd4-err_grace-should-bump-seqid-on-open	Wed Jul  6 13:08:28 2005
+++ 25-akpm/fs/nfsd/nfs4proc.c	Wed Jul  6 13:08:28 2005
@@ -169,12 +169,6 @@ nfsd4_open(struct svc_rqst *rqstp, struc
 		(int)open->op_fname.len, open->op_fname.data,
 		open->op_stateowner);
 
-	if (nfs4_in_grace() && open->op_claim_type != NFS4_OPEN_CLAIM_PREVIOUS)
-		return nfserr_grace;
-
-	if (!nfs4_in_grace() && open->op_claim_type == NFS4_OPEN_CLAIM_PREVIOUS)
-		return nfserr_no_grace;
-
 	/* This check required by spec. */
 	if (open->op_create && open->op_claim_type != NFS4_OPEN_CLAIM_NULL)
 		return nfserr_inval;
diff -puN fs/nfsd/nfs4state.c~nfsd4-err_grace-should-bump-seqid-on-open fs/nfsd/nfs4state.c
--- 25/fs/nfsd/nfs4state.c~nfsd4-err_grace-should-bump-seqid-on-open	Wed Jul  6 13:08:28 2005
+++ 25-akpm/fs/nfsd/nfs4state.c	Wed Jul  6 13:08:28 2005
@@ -1790,6 +1790,12 @@ nfsd4_process_open2(struct svc_rqst *rqs
 	struct nfs4_delegation *dp = NULL;
 	int status;
 
+	if (nfs4_in_grace() && open->op_claim_type != NFS4_OPEN_CLAIM_PREVIOUS)
+		return nfserr_grace;
+
+	if (!nfs4_in_grace() && open->op_claim_type == NFS4_OPEN_CLAIM_PREVIOUS)
+		return nfserr_no_grace;
+
 	status = nfserr_inval;
 	if (!TEST_ACCESS(open->op_share_access) || !TEST_DENY(open->op_share_deny))
 		goto out;
_
